//Basic energy monitoring sketch
//Authors: Trystan Lea, Eric Sandeen

//Licenced under GNU General Public Licence more details here
// openenergymonitor.org

//Sketch measures voltage and current. 
//and then calculates useful values like real power,
//apparent power, powerfactor, Vrms, Irms.


/* REQUIRES in the sketch directory
   rfm.ino                              // the low-level routines for the RF Modules
*/

#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif

#include <SPI.h>                        // SPI bus for the RFM module
#include <util/crc16.h>                 // Checksum 

#define RFM69CW                         // The type of Radio Module, can be RFM69CW or RFM12B, 
                                        //   or don't define anything if radio is not required - in which case 
                                        //     the serial output will be for information and debugging only.
                                        // The sketch will hang if the wrong radio module is specified, or if one is specified and not fitted.
#undef RF12_433MHZ
#undef RF12_868MHZ
#undef RF12_915MHZ                      // Should not be present, but can cause problems if they are.

#define RF12_433MHZ                     // Frequency of RFM module can be 
                                        //    RF12_433MHZ, RF12_868MHZ or RF12_915MHZ. 
                                        //  You should use the one matching the module you have.
                                        //  (Note: this is different from the normal OEM definition.)

#define RFPWR 0x9F                      // Transmitter power: 0x9F = +13 dBm (max) 
                                        //  The control range for transmitter power is 0x80 to 0x9F = -18 dBm to +13 dBm.
                                        //  NOTE: Ensure a correctly matched antenna is used when operating at or near maximum power, 
                                        //  otherwise the module may be damaged.

                                                                                                 
#define RFMSELPIN 10                    // Pins for the RFM Radio module  
                                        //   10 for the V3.4; 5 or 10 for the Shield but depending on jumper setting, 4 for the V3.2/RFu_328 V1.2
#define RFMIRQPIN 2                     // Pins for the RFM Radio module:
                                        //    2 for the V3.4; 2 or  3 for the Shield but depending on jumper setting, 3 for the V3.2/RFu_328 V1.2


const int nodeID = 11;                  //  node ID for this emonTx. This sketch does NOT interrogate the DIP switch.

const int networkGroup = 210;           //  wireless network group
                                        //  - needs to be same as emonBase and emonGLCD. OEM default is 210

typedef struct { double power, Irms, power1, Irms1; } PayloadTX;
                                        // neat way of packaging data for RF comms
                                        // Include all the variables that are desired, ensure the same struct is used to receive.
                                        // The maximum size is 60 Bytes

PayloadTX emontx;                       // create an instance


//Setup variables
int numberOfSamples = 3000;

//Set Voltage and current input pins
int inPinI = 1;
int inPinI1 = 2;

//Sanity check calibration method thanks to Eric Sandeen http://sandeen.net/wordpress
//See discussion here: http://openenergymonitor.org/emon/node/59
//Enter the values of your setup below 

// Voltage is reduced both by wall xfmr & voltage divider
#define AC_WALL_VOLTAGE        122
#define AC_ADAPTER_VOLTAGE     15.2
// Ratio of the voltage divider in the circuit
#define AC_VOLTAGE_DIV_RATIO   11
// CT: Voltage depends on current, burden resistor, and turns
#define CT_BURDEN_RESISTOR    62
#define CT_TURNS              1900

//Calibration coeficients
//These need to be set in order to obtain accurate results
//Set the above values first and then calibrate futher using normal calibration method described on how to build it page.
double VCAL = 1.0;
double VCAL1 = 1.0;
double ICAL = 2.87;
double ICAL1 = 2.88;
double PHASECAL = 2.3;

// Initial gueses for ratios, modified by VCAL/ICAL tweaks
double I_RATIO = (long double)CT_TURNS / CT_BURDEN_RESISTOR * 5 / 1024 * ICAL;
double I_RATIO1 = (long double)CT_TURNS / CT_BURDEN_RESISTOR * 5 / 1024 * ICAL1;

//Sample variables
int lastSampleI,sampleI;
int lastSampleI1,sampleI1;

//Filter variables
double lastFilteredI, filteredI;
double lastFilteredI1, filteredI1;
double filterTemp;

//Power calculation variables
double sqI,sumI;
double sqI1,sumI1;

//Useful value variables
double Irms;
double Irms1;
       
void setup()
{
   Serial.begin(9600); 
#if defined RFM12B || defined RFM69CW        
   rfm_init();
#endif

}

void loop()
{ 

for (int n=0; n<numberOfSamples; n++)
{

   //Used for offset removal
   lastSampleI=sampleI;
   lastSampleI1=sampleI1;
   
   //Read in voltage and current samples.   
   sampleI = analogRead(inPinI);
   sampleI1 = analogRead(inPinI1);
   
   //Used for offset removal
   lastFilteredI = filteredI;
   lastFilteredI1 = filteredI1;
  
   //Digital high pass filters to remove 2.5V DC offset.
   filteredI = 0.996 * (lastFilteredI+sampleI-lastSampleI);
   filteredI1 = 0.996 * (lastFilteredI1+sampleI1-lastSampleI1);
   
   //Root-mean-square method current
   //1) square current values
   sqI = filteredI * filteredI;
   sqI1 = filteredI1 * filteredI1;
   //2) sum 
   sumI += sqI;
   sumI1 += sqI1;
}

//Calculation of the root of the mean of the voltage and current squared (rms)
//Calibration coeficients applied. 
Irms = (I_RATIO * sqrt(sumI / numberOfSamples)) - 0.09;
Irms1 = (I_RATIO1 * sqrt(sumI1 / numberOfSamples)) - 0.09; 


double watt = (230 * Irms);
Serial.println("***** Solar Anlage Alberd *****");
Serial.print("Anlage 1 : ");Serial.print(watt);Serial.print(" Watt "); Serial.print(Irms);Serial.print(" Ampere");
Serial.println();
double watt1 = (230 * Irms1);
Serial.println("***** Solar Anlage Klaus *****");
Serial.print("Anlage 2 : ");Serial.print(watt1);Serial.print(" Watt "); Serial.print(Irms1);Serial.print(" Ampere");
Serial.println();
Serial.println(); //delay(3000);


#if defined RFM12B || defined RFM69CW
    emontx.power = watt;
    emontx.Irms = Irms;
    emontx.power1 = watt1;
    emontx.Irms1 = Irms1;
    rfm_send((byte *)&emontx, sizeof(emontx), networkGroup, nodeID);      // *SEND RF DATA*
#endif



//Reset accumulators
sumI = 0;
sumI1 = 0;
delay(10);

}
